// temp.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <windows.h>               //header file for windows
#include <gl\gl.h>                 //header file for openGL
#include <gl\glu.h>                //header file for the openGL utility library
#include "glut.h"               //header file for GLUT
#include <conio.h>
#include <time.h>
#include <math.h>
#include "mouse.h"

static void redraw(void);          //declarations
char frames [sizeof(long)*8+1];
long startTime;
long renders = 0;
int TICKSPEED = 2500/60;
const GLfloat PI = 3.1459;

int cameraMode = 0;

GLfloat ballRotation = 0.0f;
GLfloat ballRotationFactor = 1.1f;

bool pause = false;

GLfloat zoomFactor = 0.0f;
GLfloat rotationFactor = 0.1f;

GLfloat ballVelocity = 0.0f;
GLfloat ballPosition = 10.0f;
long simulationTime = 0;

int windowWidth = 600;
int windowHeight = 600;
const double ELASTICITY = 0.80;

/* The dimensions of the viewing frustum */
GLfloat fleft   = -1.0;
GLfloat fright  =  1.0;
GLfloat fbottom = -1.0;
GLfloat ftop    =  1.0;
GLfloat zNear   = -2.0;
GLfloat zFar    = -7.0;

GLuint sphereTexture;
GLuint planeTexture;

GLfloat angle = 0.0f;

struct Vector
{
	double x;
	double y;
	double z;
	double w;
};

GLfloat getPhysics(int time, bool updateVel)
{
	GLfloat accel = -0.00000098;

	if(updateVel)
	{
		ballVelocity += accel * time;
	}
	return ballVelocity * time;
}



// load a 256x256 RGB .RAW file as a texture
GLuint LoadTextureRAW( const char * filename, int wrap )
{
    GLuint texture;
    int width, height;
    BYTE * data;
    FILE * file;

    // open texture data
    file = fopen( filename, "rb" );
    if ( file == NULL ) return 0;

    // allocate buffer
    width = 256;
    height = 256;
    data = (BYTE*)malloc( width * height * 3 );

    // read texture data
    fread( data, width * height * 3, 1, file );
    fclose( file );

    // allocate a texture name
    glGenTextures( 1, &texture );

    // select our current texture
    glBindTexture( GL_TEXTURE_2D, texture );

    // select modulate to mix texture with color for shading
    glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );

    // when texture area is small, bilinear filter the closest mipmap
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
                     GL_LINEAR_MIPMAP_NEAREST );
    // when texture area is large, bilinear filter the first mipmap
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );

    // if wrap is true, the texture wraps over at the edges (repeat)
    //       ... false, the texture ends at the edges (clamp)
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S,
                     wrap ? GL_REPEAT : GL_CLAMP );
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T,
                     wrap ? GL_REPEAT : GL_CLAMP );

    // build our texture mipmaps
    gluBuild2DMipmaps( GL_TEXTURE_2D, 3, width, height,
                       GL_RGB, GL_UNSIGNED_BYTE, data );

    // free buffer
    free( data );

    return texture;
}


/*
   Create a sphere centered at c, with radius r, and precision n
   Draw a point for zero radius spheres
*/
void CreateSphere(double r,int n)
{

		// setup texture mapping
	glEnable( GL_TEXTURE_2D );
	glBindTexture( GL_TEXTURE_2D, sphereTexture );
   int i,j;
   double theta1,theta2,theta3;
   Vector e,p;

   if (r < 0)
      r = -r;
   if (n < 0)
      n = -n;
   if (n < 4 || r <= 0) {
      glBegin(GL_POINTS);
      glVertex3f(0.0f,0.0f,0.0f);
      glEnd();
      return;
   }

   for (j=0;j<n/2;j++) {
      theta1 = j * 2 * PI / n - PI / 2;
      theta2 = (j + 1) * 2 * PI / n - PI / 2;

      glBegin(GL_QUAD_STRIP);
      for (i=0;i<=n;i++) {
         theta3 = i * 2 * PI / n;

         e.x = cos(theta2) * cos(theta3);
         e.y = sin(theta2);
         e.z = cos(theta2) * sin(theta3);
         p.x = 0.0f + r * e.x;
         p.y = 0.0f + r * e.y;
         p.z = 0.0f + r * e.z;

         glNormal3f(e.x,e.y,e.z);
         glTexCoord2f(i/(double)n,2*(j+1)/(double)n);
         glVertex3f(p.x,p.y,p.z);

         e.x = cos(theta1) * cos(theta3);
         e.y = sin(theta1);
         e.z = cos(theta1) * sin(theta3);
         p.x = 0.0f + r * e.x;
         p.y = 0.0f + r * e.y;
         p.z = 0.0f + r * e.z;

         glNormal3f(e.x,e.y,e.z);
         glTexCoord2f(i/(double)n,2*j/(double)n);
         glVertex3f(p.x,p.y,p.z);
      }
      glEnd();
   }
}





void renderSphere()
{
	float ambient[4]  = {0.9f, 0.5f, 0.5f, 1.0f} ;
	float diffuse[4]  = {0.9f, 0.5f, 0.5f, 1.0f} ;
	float specular[4]  = {0.9f, 0.5f, 0.5f, 1.0f} ;

	glMaterialfv( GL_FRONT, GL_AMBIENT,   ambient );
	glMaterialfv( GL_FRONT, GL_DIFFUSE,   diffuse );
	glMaterialfv( GL_FRONT, GL_SPECULAR,  specular );
	glMaterialf(  GL_FRONT, GL_SHININESS, 15.0f );
	
	// SPHERE!
	glPushMatrix();
	
	glTranslatef(0.0f, ballPosition + getPhysics(simulationTime, false), 0.0f);
	glRotatef(ballRotation, 1.0f, 0.0f, 0.0f); 

	glColor3f(1.0f,0.0f,1.0f);
	
	CreateSphere(0.5, 100);

	glPopMatrix();
}

void renderBlur()
{
	//printf("%f\n", ballVelocity);
	if( ballVelocity >= 0.0003f || ballVelocity <= -0.0003f )
	{
		
		float ambient[4]  = {0.2f, 0.2f, 0.2f, 0.1f} ;
		float diffuse[4]  = {0.2f, 0.2f, 0.2f, 1.0f} ;
		float specular[4]  = {0.2f, 0.2f, 0.2f, 0.1f} ;

		glMaterialfv( GL_FRONT, GL_AMBIENT,   ambient );
		glMaterialfv( GL_FRONT, GL_DIFFUSE,   diffuse );
		glMaterialfv( GL_FRONT, GL_SPECULAR,  specular );
		glMaterialf(  GL_FRONT, GL_SHININESS, 1.0f );
		
		// SPHERE!
		glPushMatrix();
		
		glTranslatef(0.0f, ballPosition + getPhysics(simulationTime-125, false), 0.0f);
		glRotatef(ballRotation, 1.0f, 0.0f, 0.0f); 

		CreateSphere(0.5, 100);

		glPopMatrix();

		ambient[0] = ambient[1] = ambient[2] = 0.1f;
		diffuse[0] = diffuse[1] = diffuse[2] = 0.1f;
		specular[0] = specular[1] = specular[2] = 0.1f;

		glMaterialfv( GL_FRONT, GL_AMBIENT,   ambient );
		glMaterialfv( GL_FRONT, GL_DIFFUSE,   diffuse );
		glMaterialfv( GL_FRONT, GL_SPECULAR,  specular );
		glMaterialf(  GL_FRONT, GL_SHININESS, 1.0f );
		
		// SPHERE!
		glPushMatrix();
		
		glTranslatef(0.0f, ballPosition + getPhysics(simulationTime-250, false), 0.0f);
		glRotatef(ballRotation, 1.0f, 0.0f, 0.0f); 

		CreateSphere(0.5, 100);

		glPopMatrix();

		ambient[0] = ambient[1] = ambient[2] = 0.05f;
		diffuse[0] = diffuse[1] = diffuse[2] = 0.05f;
		specular[0] = specular[1] = specular[2] = 0.05f;

		glMaterialfv( GL_FRONT, GL_AMBIENT,   ambient );
		glMaterialfv( GL_FRONT, GL_DIFFUSE,   diffuse );
		glMaterialfv( GL_FRONT, GL_SPECULAR,  specular );
		glMaterialf(  GL_FRONT, GL_SHININESS, 1.0f );
		
		// SPHERE!
		glPushMatrix();
		
		glTranslatef(0.0f, ballPosition + getPhysics(simulationTime-500, false), 0.0f);
		glRotatef(ballRotation, 1.0f, 0.0f, 0.0f); 

		CreateSphere(0.5, 100);

		glPopMatrix();
	}
}


void renderPlane(int size, double p_x, double p_z)
{
	glColor3f(1.0f,1.0f,1.0f);

	float ambient[4]  = {0.1f, 0.1f, 0.1f, 1.0f} ;
	float diffuse[4]  = {1.0f, 1.0f, 1.0f, 1.0f} ;
	float specular[4]  = {1.0f, 1.0f, 1.0f, 1.0f} ;

	glMaterialfv( GL_FRONT, GL_AMBIENT,   ambient );
	glMaterialfv( GL_FRONT, GL_DIFFUSE,   diffuse );
	glMaterialfv( GL_FRONT, GL_SPECULAR,  specular );
	glMaterialf(  GL_FRONT, GL_SHININESS, 128.0f );


	// setup texture mapping
	glEnable( GL_TEXTURE_2D );
	glBindTexture( GL_TEXTURE_2D, planeTexture );

	for(int x = 0; x<size; ++x)
	{
		for(int y = 0; y<size; ++y)
		{
			glPushMatrix();
			glTranslatef(0.0f, -1.5f, 0.0f);
			glBegin( GL_QUADS );
			
			glTexCoord2d(0.0,0.0); 
			glNormal3f(0.0f, 1.0f, 0.0f);
			glVertex3d(p_x + -1.0f + x * 2, 0.0f, p_z + -1.0f + y* 2);

			glTexCoord2d(1.0,0.0); 
			glNormal3f(0.0f, 1.0f, 0.0f);
			glVertex3d(p_x +  1.0f + x* 2, 0.0f, p_z + -1.0f + y* 2);

			glTexCoord2d(1.0,1.0); 
			glNormal3f(0.0f, 1.0f, 0.0f);
			glVertex3d(p_x +  1.0f + x* 2, 0.0f, p_z + 1.0f + y* 2);

			glTexCoord2d(0.0,1.0); 
			glNormal3f(0.0f, 1.0f, 0.0f);
			glVertex3d(p_x + -1.0f + x* 2, 0.0f, p_z + 1.0f + y* 2);
			
			glEnd();
			
			glPopMatrix();
		}
	}
}


void renderShadow()
{
	float ambient[4]  = {0.0f, 0.0f, 0.0f, 1.0f} ;
	float diffuse[4]  = {0.0f, 0.0f, 0.0f, 1.0f} ;
	float specular[4]  = {0.0f, 0.0f, 0.0f, 1.0f} ;

	glMaterialfv( GL_FRONT, GL_AMBIENT,   ambient );
	glMaterialfv( GL_FRONT, GL_DIFFUSE,   diffuse );
	glMaterialfv( GL_FRONT, GL_SPECULAR,  specular );
	glMaterialf(  GL_FRONT, GL_SHININESS, 0.0f );	

	glRotatef(90.0f, 1.0f, 0.0f, 0.0f);
	glPushMatrix();
	glColor3f(0.0f,0.0f,0.0f);
	glutSolidCone( 0.5f, 0.0f, 100, 100);
	glPopMatrix();
	
}


void enableLighting()
{
	float ambient0[4]  = {0.3f, 0.3f, 0.3f, 1.0f} ;
	float diffuse0[4]  = {0.8f, 0.8f, 0.8f, 1.0f} ;
	float specular0[4]  = {0.8f, 0.8f, 0.8f, 1.0f} ;
	float position0[4]  = {0.0f, ballPosition + 15.0f, 0.0f, 1.0f} ;
	float direction0[4]  = {0.0f, -1.0f, 0.0f, 1.0f} ;

	glLightfv( GL_LIGHT0, GL_AMBIENT, ambient0 );
	glLightfv( GL_LIGHT0, GL_DIFFUSE,  diffuse0 );
	glLightfv( GL_LIGHT0, GL_SPECULAR, specular0 );
	glLightfv( GL_LIGHT0, GL_POSITION, position0 );

	glLightfv( GL_LIGHT0, GL_SPOT_DIRECTION, direction0 );
	
	GLfloat angle = 30.0f;
	glLightf( GL_LIGHT0, GL_SPOT_CUTOFF, angle );
	GLfloat exp = 128.0f;
	glLightf( GL_LIGHT0, GL_SPOT_EXPONENT, exp );

	glLightf( GL_LIGHT0, GL_CONSTANT_ATTENUATION, 1.0f );
	glEnable( GL_LIGHT0 );



	float ambient1[4]  = {0.1f, 0.1f, 0.1f, 0.0f} ;
	float diffuse1[4]  = {0.2f, 0.2f, 0.2f, 1.0f} ;
	float specular1[4]  = {0.2f, 0.2f, 0.2f, 1.0f} ;
	float position1[4]  = {0.0f, 40.0f, 0.0f, 1.0f} ;

	glLightfv( GL_LIGHT1, GL_AMBIENT, ambient1 );
	glLightfv( GL_LIGHT1, GL_DIFFUSE,  diffuse1 );
	glLightfv( GL_LIGHT1, GL_SPECULAR, specular1 );
	glLightfv( GL_LIGHT1, GL_POSITION, position1 );

	glEnable( GL_LIGHT1 );
/*

	float ambient2[4]  = {0.0f, 0.0f, 0.0f, 0.0f} ;
	float diffuse2[4]  = {1.0f, 1.0f, 1.0f, 1.0f} ;
	float specular2[4]  = {1.0f, 1.0f, 1.0f, 1.0f} ;
	float position2[4]  = {10.0f, 1.0f, 10.0f, 1.0f} ;

	glLightfv( GL_LIGHT2, GL_AMBIENT, ambient2 );
	glLightfv( GL_LIGHT2, GL_DIFFUSE,  diffuse2 );
	glLightfv( GL_LIGHT2, GL_SPECULAR, specular2 );
	glLightfv( GL_LIGHT2, GL_POSITION, position2 );

	glEnable( GL_LIGHT2 );

	float ambient3[4]  = {0.0f, 0.0f, 0.0f, 0.0f} ;
	float diffuse3[4]  = {0.0f, 0.0f, 1.0f, 1.0f} ;
	float specular3[4]  = {0.0f, 0.0f, 1.0f, 1.0f} ;
	float position3[4]  = {-10.0f, 1.0f, -10.0f, 1.0f} ;

	glLightfv( GL_LIGHT3, GL_AMBIENT, ambient3 );
	glLightfv( GL_LIGHT3, GL_DIFFUSE,  diffuse3 );
	glLightfv( GL_LIGHT3, GL_SPECULAR, specular3 );
	glLightfv( GL_LIGHT3, GL_POSITION, position3 );

	glEnable( GL_LIGHT3 );
*/
	glEnable( GL_LIGHTING );
}

void disableLighting()
{
	glDisable( GL_LIGHT0 );
	glDisable( GL_LIGHT1 );
	glDisable( GL_LIGHT2 );
	glDisable( GL_LIGHT3 );
	glDisable( GL_LIGHTING );
}

void setOrthographicProjection() {

	// switch to projection mode
	glMatrixMode(GL_PROJECTION);
	// save previous matrix which contains the 
	//settings for the perspective projection
	glPushMatrix();
	// reset matrix
	glLoadIdentity();
	// set a 2D orthographic projection
	gluOrtho2D(0, windowWidth, 0, windowHeight);
	// invert the y axis, down is positive
	glScalef(1, -1, 1);
	// mover the origin from the bottom left corner
	// to the upper left corner
	glTranslatef(0, -windowHeight, 0);
	glMatrixMode(GL_MODELVIEW);
}

void resetPerspectiveProjection() {
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
}

void renderSpacedBitmapString(
			float x, 
			float y,
			int spacing, 
			void *font,
			char *string) {
  char *c;
  int x1=x;
  for (c=string; *c != '\0'; c++) {
	glRasterPos2f(x1,y);
    	glutBitmapCharacter(font, *c);
	x1 = x1 + glutBitmapWidth(font,*c) + spacing;
  }
}

/*
 * The main drawing routine.  Based on the current display mode, other
 * helper functions may be called.
 */
void myDisplay (void) {

	// Calculate the fps.


	// The camera follows the ball.
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective( 45.0f, (double)windowWidth/windowHeight, 1, 100 );
	glViewport( 0, 0, windowWidth, windowHeight );

	switch(cameraMode)
	{
		case 0:
			gluLookAt(0.0f, 1.0f, 7.5f + zoomFactor, 0.0f, ballPosition, 0.0f, 0.0f, 1.0f, 0.0f);
			glRotatef(angle, 0.0f, 1.0f, 0.0f); 
			break;
		case 1:
			gluLookAt(0.0f, ballPosition, 7.5f + zoomFactor, 0.0f, ballPosition, 0.0f, 0.0f, 1.0f, 0.0f);
			glRotatef(angle, 0.0f, 1.0f, 0.0f); 
			break;
		case 2:
			gluLookAt(0.0f, ballPosition + 7.5f + zoomFactor, 0.0f, 0.0f, ballPosition, 0.0f, 0.0f, 0.0f, 1.0f);
			glRotatef(angle, 0.0f, 1.0f, 0.0f); 
			break;
		case 3:
			gluLookAt(7.5f + zoomFactor, ballPosition + 7.5f + zoomFactor, 7.5f + zoomFactor, 0.0f, ballPosition, 0.0f, 0.0f, 1.0f, 0.0f);
			glRotatef(angle, 0.0f, 1.0f, 0.0f);
			break;
	}
/* Clear the pixels (aka colors) and the z-buffer */
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glMatrixMode(GL_MODELVIEW);	/* All matrix operations are for the model */
	glLoadIdentity();
	
	

	
	enableLighting();

	glEnable(GL_DEPTH_TEST);	/* Use the Z - buffer for visibility */

	
	
	//enableLighting();	

	renderPlane(24, -20, -20);


	renderSphere();
	glLoadIdentity();
	glTranslatef(0.0f, -1.49f, 0.0f);
	glScalef( (0.05)*(ballPosition - (-1.5f)) + 1.0, 0.0f, (0.05)*(ballPosition - (-1.5f)) + 1.0);

	renderShadow();

	glColor4f(0.1f, 0.1f, 0.1f, 0.1f);   //defines the alpha value = 0.25f
   

	glEnable (GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_SRC_ALPHA);  

	glLoadIdentity();

	renderBlur();

	glDisable(GL_BLEND);	

	disableLighting();
	setOrthographicProjection();
	glPushMatrix();
	glLoadIdentity();
	glColor3f(1.0f,1.0f,1.0f);
	renderSpacedBitmapString(5,15, 0, GLUT_BITMAP_HELVETICA_12, "Michael Chrien");
	renderSpacedBitmapString(5,30, 0, GLUT_BITMAP_HELVETICA_10, "Frames Per Second:");
	renderSpacedBitmapString(100,30, 0, GLUT_BITMAP_HELVETICA_10, frames);

	renderSpacedBitmapString(5,40, 0, GLUT_BITMAP_HELVETICA_10, "'C' changes camera mode");
	renderSpacedBitmapString(5,50, 0, GLUT_BITMAP_HELVETICA_10, "'P' pauses ball");
	renderSpacedBitmapString(5,60, 0, GLUT_BITMAP_HELVETICA_10, "'W'/'E' alter rotation speed");
	renderSpacedBitmapString(5,70, 0, GLUT_BITMAP_HELVETICA_10, "'A'/'Z' zoom in and out");
	renderSpacedBitmapString(5,80, 0, GLUT_BITMAP_HELVETICA_10, "'R' reset simulation");
	renderSpacedBitmapString(5,90, 0, GLUT_BITMAP_HELVETICA_10, "'B' apply slight upward velocity");
	renderSpacedBitmapString(5,100, 0, GLUT_BITMAP_HELVETICA_10, "'-'/'+' alter time step");
	glPopMatrix();
	resetPerspectiveProjection();


	glFlush();	/* Flush all executed OpenGL ops finish */
	
	/*
	 * Since we are using double buffers, we need to call the swap
	 * function every time we are done drawing.
	 */

	glutSwapBuffers();

	long elapsed = time (NULL) - startTime;
	++renders;
	if (elapsed!=0)
	{
		long fps = renders/elapsed;
		ltoa(fps, frames, 10);	
		if(renders == 5)
		{
			startTime = elapsed + startTime;
			renders = 0;
		}
	}

	glutPostRedisplay();
}

/*
 * Performs specific initializations for this program (as opposed to
 * glut initialization.
 */
void myInit (int argc, char **argv) 
{

	/* Set up a black background */
	glClearColor(0.1, 0.1, 0.1, 0.0);
	glEnable( GL_TEXTURE_2D );
	//glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT); 
	//glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	//Set up our viewport such that coordinates are specified.
	//glViewport(0, 0, windowWidth, windowHeight);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	gluPerspective( 45.0f, (double)windowWidth/windowHeight, 1, 100 );
	glViewport( 0, 0, windowWidth, windowHeight );


	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	planeTexture = LoadTextureRAW( "concrete.raw", TRUE );
	sphereTexture = LoadTextureRAW( "basketball.raw", TRUE );
	startTime = time (NULL);

}

double velocity = 0;
double lastVelocity = 0;
double startPos = -1.0;
double origPos = -1.0;
bool reset = false;
//Increases the circles' size!


void myResize(int x, int y)
{
	windowWidth = x;
	windowHeight = y;
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	gluPerspective( 45.0f, (double)windowWidth/windowHeight, 1, 100 );
	glViewport( 0, 0, windowWidth, windowHeight );


	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}



void tick(int i)
{
	angle += rotationFactor;
	
	if(!pause)
	{
		ballRotation += ballRotationFactor;
		//simulationTime = i;
		ballPosition += getPhysics(simulationTime, true);
		if(ballPosition <= -1.0f && ballVelocity >= -0.0003 && ballVelocity <= 0.0003 )
		{
			ballVelocity = 0.0f;
			simulationTime = 0;
			ballPosition = -1.0f;
			ballRotation -= ballRotationFactor;
			glutPostRedisplay();
			glutTimerFunc( TICKSPEED, tick, i);
		}
		else if(ballPosition <= -1.0f)
		{
			printf("%f\n", ballVelocity);
			ballPosition = -0.99f;
			ballVelocity = -ballVelocity * ELASTICITY;
			++simulationTime;
			ballRotation += -ballRotationFactor * ELASTICITY;
			glutPostRedisplay();
			glutTimerFunc( TICKSPEED, tick, ++i);
		}
		else
		{
			++simulationTime;
			glutPostRedisplay();
			glutTimerFunc( TICKSPEED, tick, ++i);
		}

		
	}
	else
	{
		glutPostRedisplay();
		glutTimerFunc( TICKSPEED, tick, i);
	}
}

/* Handle user input */
void myKeyHandler(unsigned char ch, int x, int y) {
	switch(ch) {
		case 'C':
		case 'c':
			++cameraMode;
			if(cameraMode == 4)
				cameraMode = 0;
			break;


		case 'b':
		case 'B':
			//if(reset == false)
			//{
				simulationTime = 0;
				ballVelocity += 0.002;
				//ballPosition = -0.99f;
				//velocity += 1000;	
				//startPos = myPos.y; 
				//reset = true;
			//}
			//printf("Camera reset.\n");
			break;
		case 'a':
		case 'A':
			zoomFactor -= 0.1f;
			break;
		case 'z':
		case 'Z':
			zoomFactor += 0.1f;
			break;

		case 'W':
		case 'w':
			rotationFactor -= 0.1f;
			break;
		case 'E':
		case 'e':
			rotationFactor += 0.1f;
			break;
		case 'R':
		case 'r':
			ballPosition = 20.0f;
			ballVelocity = 0.00002f;
			simulationTime = 0;
			break;
		case 'P':
		case 'p':
			pause = !pause;
			break;


		case '-':
		case '_':
			if(TICKSPEED < 81)
			{
				TICKSPEED += (100/60);
				printf("Speed: %d\n", TICKSPEED);
			}
			else
				printf("Slow as possible!\n");
			break;
		case '+':
		case '=':
			if(TICKSPEED > 200/60)
			{
				TICKSPEED -= (100/60);
				printf("Speed: %d\n", TICKSPEED);
			}
			else
				printf("Fast as possible!\n");
			break;
		default:
			/* Unrecognized key press, just return */
			return;

			break;
	}

	/*
	 * If control reaches here, the key press was recognized.  Refresh
	 * the screen, since most key presses change the display in some way.
	 */
	glutPostRedisplay();

	return;
}

int _tmain(int argc, _TCHAR* argv[])
{
	glutInit(&argc, argv);

	/* Set initial window size and screen offset */
	glutInitWindowSize(windowWidth, windowHeight);
	glutInitWindowPosition(50, 50);

	/* Using: RGB (no alpha), double buffering, z-buffer */
	glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);

	glutCreateWindow("Ball!");

	/* Set the function callbacks */
	glutDisplayFunc(myDisplay);
	glutReshapeFunc(myResize);
	glutKeyboardFunc(myKeyHandler);
	glutMouseFunc(myMouseButton);
	glutMotionFunc(myMouseMotion);
	//glutTimerFunc(1000, updateCircles, 1);
	glutTimerFunc( TICKSPEED, tick, 1);
	/* User specific initialization */
	myInit(argc, argv);
	
	glutMainLoop();	/* Go into the main glut control loop, will not return */

	/* Control flow will never reach here */
	return 0;
}
